import sys

from workspace import command_build, command_cmake, command_help, command_install, command_pull, command_status, command_graph

commands = [command_build, command_cmake, command_help, command_install, command_pull, command_status, command_graph]
